# num of kafka partitions
NUM_OF_PARTITION = 3

SERVICE_INPUT_TOPIC = 'service-input-event'
SERVICE_OUTPUT_TOPIC = 'service-output-event'
SERVICE_OBSERVE_TOPIC = 'service-observed-event'
RECONFIG_TOPIC = 'reconfig-event'
CONTROL_TOPIC = "control-event"
IS_ALIVE = "healthy"
ACTIVE_SERVICES_UPDATED_TOPIC = "active-services-updated-event"
AGENT_EVENTS = "agent-result-event"
# machine topics
MACHINE_EVENTS = 'machine-set-event'
SIMULATION_EVENTS = 'simulation-event'
PROPOSAL_TOPIC = 'proposal-event'
